@@ -6,6 +6,7 @@ use std::path::{Path, PathBuf};
 use std::time::Duration;
 
 use once_cell::unsync::OnceCell;
+use ureq::tls::{RootCerts, TlsConfig};
 use yansi::Paint;
 use zip::ZipArchive;
 
@@ -45,7 +46,7 @@ impl<'a> Cache<'a> {
 
     /// Send a GET request with the provided agent and return the response body.
     fn get_asset(agent: &ureq::Agent, url: &str) -> Result<Vec<u8>> {
-        info_start!("downloading '{}'... ", url.split('/').last().unwrap());
+        info_start!("downloading '{}'... ", url.split('/').next_back().unwrap());
 
         let mut resp = match agent.get(url).call() {
             Ok(r) => r,
@@ -83,6 +84,11 @@ impl<'a> Cache<'a> {
         let agent = ureq::Agent::config_builder()
             .user_agent(USER_AGENT)
             .timeout_global(Some(Duration::from_secs(5)))
+            .tls_config(
+                TlsConfig::builder()
+                    .root_certs(RootCerts::PlatformVerifier)
+                    .build(),
+            )
             .build()
             .into();
 
