@@ -13,44 +13,44 @@ use crate::cache::Cache;
 use crate::error::{Error, ErrorKind, Result};
 use crate::util::{self, warnln};
 
-const HEX_ERR: &str = "6 hexadecimal digits, optionally prefixed with '#'";
-
-struct HexColorVisitor;
-impl Visitor<'_> for HexColorVisitor {
-    type Value = [u8; 3];
-
-    fn expecting(&self, formatter: &mut std::fmt::Formatter) -> std::fmt::Result {
-        formatter.write_str(HEX_ERR)
-    }
-
-    fn visit_str<E>(self, v: &str) -> std::result::Result<Self::Value, E>
-    where
-        E: serde::de::Error,
-    {
-        let hex = v.strip_prefix('#').unwrap_or(v);
-
-        if hex.len() != 6 {
-            return Err(serde::de::Error::invalid_length(hex.len(), &HEX_ERR));
-        }
-
-        let invalid_val = |_| serde::de::Error::invalid_value(Unexpected::Str(v), &HEX_ERR);
-        let r = u8::from_str_radix(&hex[0..2], 16).map_err(invalid_val)?;
-        let g = u8::from_str_radix(&hex[2..4], 16).map_err(invalid_val)?;
-        let b = u8::from_str_radix(&hex[4..6], 16).map_err(invalid_val)?;
-
-        Ok([r, g, b])
-    }
-}
-
 fn hex_to_rgb<'de, D>(deserializer: D) -> std::result::Result<[u8; 3], D::Error>
 where
     D: Deserializer<'de>,
 {
+    const HEX_ERR: &str = "6 hexadecimal digits, optionally prefixed with '#'";
+
+    struct HexColorVisitor;
+    impl Visitor<'_> for HexColorVisitor {
+        type Value = [u8; 3];
+
+        fn expecting(&self, formatter: &mut std::fmt::Formatter) -> std::fmt::Result {
+            formatter.write_str(HEX_ERR)
+        }
+
+        fn visit_str<E>(self, v: &str) -> std::result::Result<Self::Value, E>
+        where
+            E: serde::de::Error,
+        {
+            let hex = v.strip_prefix('#').unwrap_or(v);
+
+            if hex.len() != 6 {
+                return Err(serde::de::Error::invalid_length(hex.len(), &HEX_ERR));
+            }
+
+            let invalid_val = |_| serde::de::Error::invalid_value(Unexpected::Str(v), &HEX_ERR);
+            let r = u8::from_str_radix(&hex[0..2], 16).map_err(invalid_val)?;
+            let g = u8::from_str_radix(&hex[2..4], 16).map_err(invalid_val)?;
+            let b = u8::from_str_radix(&hex[4..6], 16).map_err(invalid_val)?;
+
+            Ok([r, g, b])
+        }
+    }
+
     deserializer.deserialize_str(HexColorVisitor)
 }
 
 #[derive(Serialize, Deserialize, Default, Clone, Copy)]
-#[serde(rename_all = "lowercase")]
+#[serde(rename_all = "snake_case")]
 pub enum OutputColor {
     Black,
     Red,
@@ -62,6 +62,14 @@ pub enum OutputColor {
     White,
     #[default]
     Default,
+    BrightBlack,
+    BrightRed,
+    BrightGreen,
+    BrightYellow,
+    BrightBlue,
+    BrightMagenta,
+    BrightCyan,
+    BrightWhite,
     Color256(u8),
     Rgb([u8; 3]),
     #[serde(deserialize_with = "hex_to_rgb")]
@@ -80,6 +88,14 @@ impl From<OutputColor> for yansi::Color {
             OutputColor::Cyan => Color::Cyan,
             OutputColor::White => Color::White,
             OutputColor::Default => Color::Primary,
+            OutputColor::BrightBlack => Color::BrightBlack,
+            OutputColor::BrightRed => Color::BrightRed,
+            OutputColor::BrightGreen => Color::BrightGreen,
+            OutputColor::BrightYellow => Color::BrightYellow,
+            OutputColor::BrightBlue => Color::BrightBlue,
+            OutputColor::BrightMagenta => Color::BrightMagenta,
+            OutputColor::BrightCyan => Color::BrightCyan,
+            OutputColor::BrightWhite => Color::BrightWhite,
             OutputColor::Color256(c) => Color::Fixed(c),
             OutputColor::Rgb(rgb) | OutputColor::Hex(rgb) => Color::Rgb(rgb[0], rgb[1], rgb[2]),
         }
@@ -233,6 +249,16 @@ impl Default for CacheConfig {
     }
 }
 
+/// Defines which options should be shown in short|long placeholders (`{{[ ... ]}}`).
+#[derive(Serialize, Deserialize, PartialEq, Default)]
+#[serde(rename_all = "lowercase")]
+pub enum OptionStyle {
+    Short,
+    #[default]
+    Long,
+    Both,
+}
+
 #[derive(Serialize, Deserialize)]
 #[serde(deny_unknown_fields, default)]
 pub struct OutputConfig {
@@ -244,8 +270,12 @@ pub struct OutputConfig {
     pub show_hyphens: bool,
     /// Show a custom string instead of a hyphen.
     pub example_prefix: Cow<'static, str>,
+    /// Set the max line length. 0 means to use the terminal width.
+    pub line_length: usize,
     /// Strip empty lines from pages.
     pub compact: bool,
+    /// Display the specified options in pages wherever possible.
+    pub option_style: OptionStyle,
     /// Print pages in raw markdown.
     pub raw_markdown: bool,
 }
@@ -257,7 +287,9 @@ impl Default for OutputConfig {
             platform_title: false,
             show_hyphens: false,
             example_prefix: Cow::Borrowed("- "),
+            line_length: 0,
             compact: false,
+            option_style: OptionStyle::default(),
             raw_markdown: false,
         }
     }
