@@ -1,12 +1,15 @@
 use std::borrow::Cow;
+use std::fmt::Write as _;
 use std::fs::File;
 use std::io::{self, BufRead, BufReader, BufWriter, Write};
 use std::path::{Path, PathBuf};
 use std::sync::atomic::Ordering::Relaxed;
 
+use terminal_size::terminal_size;
+use unicode_width::UnicodeWidthStr;
 use yansi::{Paint, Style};
 
-use crate::config::Config;
+use crate::config::{Config, OptionStyle};
 use crate::error::{Error, ErrorKind, Result};
 use crate::util::{warnln, PagePathExt};
 
@@ -36,22 +39,44 @@ pub struct PageRenderer<'a> {
     current_line: String,
     /// The line number of the current line.
     lnum: usize,
+    /// Max line length.
+    max_len: Option<usize>,
     /// Style configuration.
     style: RenderStyles,
     /// Other options.
     cfg: &'a Config,
 }
 
+/// Write a `yansi::Painted` to a `String`.
+///
+/// This is used to append something to a `String` without creating `String`s for every part of a
+/// line that's highlighted using a different style.
+macro_rules! write_paint {
+    ($buf:expr, $what:expr) => {
+        // This will never return an error, we're writing to a `String`.
+        let _ = write!($buf, "{}", $what);
+    };
+}
+
+/// Type of the line.
+/// Does not include types that aren't wrapped (i.e. title, empty lines and examples).
+#[derive(Clone, Copy, PartialEq)]
+enum LineType {
+    Desc,
+    Bullet,
+}
+
 impl<'a> PageRenderer<'a> {
     fn hl_code(&self, s: &str, style_normal: Style) -> String {
         let split: Vec<&str> = s.split('`').collect();
+        let mut buf = String::with_capacity(s.len());
+
         // Highlight beginning not found.
         if split.len() == 1 {
-            return s.paint(style_normal).to_string();
+            write_paint!(buf, s.paint(style_normal));
+            return buf;
         }
 
-        let mut buf = String::new();
-
         for (i, part) in split.into_iter().enumerate() {
             // Only odd indexes contain the part to be highlighted.
             // "aa `bb` cc `dd` ee"
@@ -61,9 +86,9 @@ impl<'a> PageRenderer<'a> {
             // 3: "dd"      (highlighted)
             // 4: " ee"
             if i % 2 == 0 {
-                buf += &part.paint(style_normal).to_string();
+                write_paint!(buf, part.paint(style_normal));
             } else {
-                buf += &part.paint(self.style.inline_code).to_string();
+                write_paint!(buf, part.paint(self.style.inline_code));
             }
         }
 
@@ -72,13 +97,14 @@ impl<'a> PageRenderer<'a> {
 
     fn hl_url(&self, s: &str, style_normal: Style) -> String {
         let split: Vec<&str> = s.split("<http").collect();
+        let mut buf = String::with_capacity(s.len());
+
         // Highlight beginning not found.
         if split.len() == 1 {
-            return s.paint(style_normal).to_string();
+            write_paint!(buf, s.paint(style_normal));
+            return buf;
         }
 
-        let mut buf = String::new();
-
         for part in split {
             if part.contains('>') {
                 // The first part of the second split contains the part to be highlighted.
@@ -91,11 +117,11 @@ impl<'a> PageRenderer<'a> {
 
                 // "<http" is used to detect URLs. It must be added back.
                 let hl = format!("http{}", part_split.0);
-                buf += &hl.paint(self.style.url).to_string();
-                buf += &part_split.1.paint(style_normal).to_string();
+                write_paint!(buf, hl.paint(self.style.url));
+                write_paint!(buf, part_split.1.paint(style_normal));
             } else {
                 // Highlight ending not found.
-                buf += &part.paint(style_normal).to_string();
+                write_paint!(buf, part.paint(style_normal));
             }
         }
 
@@ -104,13 +130,14 @@ impl<'a> PageRenderer<'a> {
 
     fn hl_placeholder(&self, s: &str, style_normal: Style) -> String {
         let split: Vec<&str> = s.split("{{").collect();
+        let mut buf = String::with_capacity(s.len());
+
         // Highlight beginning not found.
         if split.len() == 1 {
-            return s.paint(style_normal).to_string();
+            write_paint!(buf, s.paint(style_normal));
+            return buf;
         }
 
-        let mut buf = String::new();
-
         for part in split {
             if part.contains("}}") {
                 // The first part of the second split contains the part to be highlighted.
@@ -125,19 +152,131 @@ impl<'a> PageRenderer<'a> {
                 // This is required for special cases with three closing curly braces ("}}}").
                 // The first brace is inside the placeholder, and the last two mark the end of it.
                 let idx = part.rmatch_indices("}}").last().unwrap().0;
-                let part_split = part.split_at(idx);
+                let (inside, outside) = part.split_at(idx);
 
-                buf += &part_split.0.paint(self.style.placeholder).to_string();
-                buf += &part_split.1[2..].paint(style_normal).to_string();
+                // Select the long or short option.
+                // Skip if the user wants to display both or if the placeholder doesn't contain
+                // option selection (`[-s|--long]`).
+                if self.cfg.output.option_style != OptionStyle::Both
+                    && inside.starts_with('[')
+                    && inside.ends_with(']')
+                    && inside.contains('|')
+                {
+                    let (short, long) = inside.split_once('|').unwrap();
+                    // A single option will be displayed, using the normal style (static part).
+                    if self.cfg.output.option_style == OptionStyle::Short {
+                        // Cut out the leading `[`.
+                        write_paint!(buf, &short[1..].paint(style_normal));
+                    } else {
+                        // Cut out the trailing `]`.
+                        write_paint!(buf, &long[..long.len() - 1].paint(style_normal));
+                    }
+                } else {
+                    // Both options will be displayed, or this isn't an option placeholder.
+                    // The placeholder style is used in both cases.
+                    write_paint!(buf, inside.paint(self.style.placeholder));
+                }
+
+                // `outside` begins with "}}". We need to cut that out.
+                write_paint!(buf, &outside[2..].paint(style_normal));
             } else {
                 // Highlight ending not found.
-                buf += &part.paint(style_normal).to_string();
+                write_paint!(buf, part.paint(style_normal));
             }
         }
 
         buf
     }
 
+    /// Split the line into multiple lines if it's longer than the configured max length.
+    fn splitln(&self, s: &'a str, indent: &str, ltype: LineType) -> Cow<'a, str> {
+        let Some(max_len) = self.max_len else {
+            // We don't have the max length. Just print the entire line then.
+            return Cow::Borrowed(s);
+        };
+
+        let len_indent = indent.len();
+        let prefix_width = if ltype == LineType::Bullet && self.cfg.output.show_hyphens {
+            self.cfg.output.example_prefix.width()
+        } else {
+            0
+        };
+        let base_width = len_indent + prefix_width;
+
+        if base_width + s.width() <= max_len {
+            // The line is shorter than the max length. There is nothing to do.
+            return Cow::Borrowed(s);
+        }
+
+        let words = s.split(' ');
+        let len_s = s.len();
+        let mut cur_width = base_width;
+        //                  current_len + base_width * amount of added newlines
+        let mut buf = String::with_capacity(len_s + base_width * (len_s / max_len));
+
+        // If the example prefix is set, we need more whitespace at the beginning of the next line.
+        let indent = if prefix_width == 0 {
+            Cow::Borrowed(indent)
+        } else {
+            Cow::Owned(" ".repeat(prefix_width) + indent)
+        };
+
+        // Is the current word highlighted (inside backticks)?
+        let mut inside_hl = false;
+
+        let style_normal = match ltype {
+            LineType::Desc => self.style.desc,
+            LineType::Bullet => self.style.bullet,
+        };
+
+        for w in words {
+            let mut w_width = w.width();
+            let backtick_count = w.chars().filter(|x| *x == '`').count();
+            // Backticks won't be displayed.
+            w_width -= backtick_count;
+
+            // current + word + space after the word
+            if cur_width + w_width + 1 > max_len && cur_width != base_width {
+                // If the next word is added, the line will be longer than the configured line
+                // length.
+                //
+                // We need to add a newline + indentation, and reset the current length.
+                if yansi::is_enabled() {
+                    // Style reset. Without this, whitespace will have a background color (if one
+                    // is set).
+                    let _ = style_normal.fmt_suffix(&mut buf);
+                }
+                buf.push('\n');
+                buf += &indent;
+                if yansi::is_enabled() {
+                    // Reenable the style.
+                    let _ = if inside_hl {
+                        self.style.inline_code.fmt_prefix(&mut buf)
+                    } else {
+                        style_normal.fmt_prefix(&mut buf)
+                    };
+                }
+                cur_width = base_width;
+            } else if cur_width != base_width {
+                // If this isn't the beginning of the line, add a space after the word.
+                buf.push(' ');
+                cur_width += 1;
+            }
+
+            buf += w;
+            cur_width += w_width;
+
+            // If there are two backticks in `w`, then `w` contains all the highlighted text.
+            // If there is only one (e.g `ab cd` => split into words ["`ab", "cd`"]), it
+            // starts/ends the highlight.
+            if backtick_count == 1 {
+                inside_hl = !inside_hl;
+            }
+        }
+
+        Cow::Owned(buf)
+    }
+
     /// Print or render the page according to the provided config.
     pub fn print(path: &'a Path, cfg: &'a Config) -> Result<()> {
         let mut page = File::open(path)
@@ -156,6 +295,11 @@ impl<'a> PageRenderer<'a> {
             stdout: BufWriter::new(io::stdout().lock()),
             current_line: String::new(),
             lnum: 0,
+            max_len: if cfg.output.line_length == 0 {
+                terminal_size().map(|x| x.0 .0 as usize)
+            } else {
+                Some(cfg.output.line_length)
+            },
             style: RenderStyles {
                 title: cfg.style.title.into(),
                 desc: cfg.style.description.into(),
@@ -213,8 +357,9 @@ impl<'a> PageRenderer<'a> {
             .reader
             .read_line(&mut self.current_line)
             .map_err(|e| Error::new(format!("'{}': {e}", self.path.display())))?;
-        self.current_line
-            .truncate(self.current_line.trim_end().len());
+        let len = self.current_line.trim_end().len();
+        self.current_line.truncate(len);
+
         Ok(n)
     }
 
@@ -245,14 +390,11 @@ impl<'a> PageRenderer<'a> {
 
     /// Write the current line to the page buffer as a description.
     fn add_desc(&mut self) -> Result<()> {
-        let desc = self.hl_code(
-            &self.hl_url(
-                self.current_line.strip_prefix(DESC).unwrap(),
-                self.style.desc,
-            ),
-            self.style.desc,
-        );
         let indent = " ".repeat(self.cfg.indent.description);
+        let line = self.current_line.strip_prefix(DESC).unwrap();
+        let line = self.splitln(line, &indent, LineType::Desc);
+        let desc = self.hl_code(&self.hl_url(&line, self.style.desc), self.style.desc);
+
         writeln!(self.stdout, "{indent}{desc}")?;
 
         Ok(())
@@ -260,17 +402,17 @@ impl<'a> PageRenderer<'a> {
 
     /// Write the current line to the page buffer as a bullet point.
     fn add_bullet(&mut self) -> Result<()> {
-        let line = if self.cfg.output.show_hyphens {
-            self.current_line
-                .replace_range(..2, &self.cfg.output.example_prefix);
-            &self.current_line
-        } else {
-            self.current_line.strip_prefix(BULLET).unwrap()
-        };
-
-        let bullet = self.hl_code(&self.hl_url(line, self.style.bullet), self.style.bullet);
         let indent = " ".repeat(self.cfg.indent.bullet);
-        writeln!(self.stdout, "{indent}{bullet}")?;
+        let line = self.current_line.strip_prefix(BULLET).unwrap();
+        let line = self.splitln(line, &indent, LineType::Bullet);
+
+        let bullet = self.hl_code(&self.hl_url(&line, self.style.bullet), self.style.bullet);
+        write!(self.stdout, "{indent}")?;
+        if self.cfg.output.show_hyphens {
+            let prefix = self.cfg.output.example_prefix.paint(self.style.bullet);
+            write!(self.stdout, "{prefix}")?;
+        }
+        writeln!(self.stdout, "{bullet}")?;
 
         Ok(())
     }
@@ -284,6 +426,7 @@ impl<'a> PageRenderer<'a> {
             .replace("\\{\\{", " \\{\\{ ")
             .replace("\\}\\}", " \\}\\} ");
 
+        let indent = " ".repeat(self.cfg.indent.example);
         let line = self
             .current_line
             .strip_prefix(EXAMPLE)
@@ -300,7 +443,6 @@ impl<'a> PageRenderer<'a> {
             .replace(" \\{\\{ ", "{{")
             .replace(" \\}\\} ", "}}");
 
-        let indent = " ".repeat(self.cfg.indent.example);
         writeln!(self.stdout, "{indent}{example}")?;
 
         Ok(())
